define(function(){
  
    var EventEmitter = function(){
        if (!(this instanceof EventEmitter)){
            return new EventEmitter();
        }
        this.listeners = {};
        this.on = this.addListener;
    };
    
    EventEmitter.prototype = {
        emit: function( type ){
            var args = Array.prototype.splice.apply(arguments, [1, arguments.length-1]);
            var listenerArray = this.listeners[type];
            if (listenerArray){
                for (var x = 0 ; x < listenerArray.length; x++){
                    listenerArray[x].apply(null, args);
                }
            }
            return this;
        },
        addListener: function(type, callback){
            if (!this.listeners[type]){
                this.listeners[type] = [];
            }
            
            this.listeners[type].push(callback);
            return this;
        },
        once:function(type, callback){
            var eventEmitter = this;
            var once = function(){
                callback.apply(null, arguments);
                eventEmitter.removeListener(type, once);
            };
            
            this.addListener(type, once);
            return this;
        },
        removeListener: function( type, callback ){
            if (callback && this.listeners[type]){
                var listenerArray = this.listeners[type];
                for (var x = 0 ; x < listenerArray.length; x++ ){
                    if (listenerArray[x] === callback){
                        listenerArray.splice(x,1);
                        x--;
                    }
                }
            } else {
                this.listeners[type] = [];
            }
            return this;
        }
    };
    
    return EventEmitter;
    
    
});