define(['EventEmitter','http','util'], function( EventEmitter, http, util ){
    
    //URL Class start.
    var URL = function( url ){
        var self = this;

        if (!(self instanceof URL)){
            return new URL(url);
        }

        var init = function( url ){

            //Thanks Douglas Crockford.
            var parse_url = /^(?:([A-Za-z]+):)?(\/{0,3})([0-9.\-A-Za-z]+)(?::(\d+))?(?:\/([^?#]*))?(?:\?([^#]*))?(?:#(.*))?$/;
            var result = parse_url.exec(url);
            self.__defineGetter__("href", function(){
                return result[0];
            });

            self.__defineGetter__("scheme", function(){
                return result[1];
            });

            self.__defineGetter__("slash", function(){
                return result[2];
            });

            self.__defineGetter__("host", function(){
                return result[3];
            });

            self.__defineGetter__("port", function(){
                return result[4] || 80;
            });

            self.__defineGetter__("path", function(){
                return result[5];
            });

            self.__defineGetter__("query", function(){
                return result[6];
            });

            self.__defineGetter__("hash", function(){
                return result[7];
            });

            self.__defineGetter__("page", function(){
                var tmpArray;
                if (result[5]){
                    tmpArray = result[5].split("/");
                    return tmpArray[tmpArray.length-1];
                } else {
                    return undefined;
                }
            });

            self.__defineGetter__("extension", function(){
                var page = self.page;
                if (page){
                    var regExp = /\.[^\.]*?$/i;
                    return page.match(regExp);
                } else {
                    return undefined;
                }
            });

        };

        if ( url ){
            init( url );
        }

        self.initialize = function( url ){
            if (!self.url && url){
                init(url);
            }
        };

        return this;
    };
    
    var EventServer = function(){
        var server = null;
        var eventServer = this;
        
        var listeners = this.listeners = {};
        eventServer.__defineGetter__('listeners', function(){
            return listeners;
        });
        
        var Event = function( type ){
            if (!(this instanceof Event)){
                return new Event( type );
            }
            
            var event = this;
            var propagation = true;
            
            if (!type){
                throw new Error("No type was provided for the event.");
            }
            
            this.stopPropagation = function(){
                propagation = false;
            };
            
            this.__defineGetter__('type', function(){
                return type;
            });
            this.__defineGetter__('propagation', function(){
                return propagation;
            });
            
            this.emit = function(){
                eventServer.emit(this);
            };
        }
        
        this.Event = Event;
        
        this.emit = function(event){
            // console.log("event type \""+event.type+"\" emitted.");
            if (event instanceof Event){
                var listenerArray = listeners[event.type];
                if (listenerArray){
                    for (var x = 0 ; x < listenerArray.length; x++){
                        if (event.propagation){
                            if (listenerArray[x](event) === false){
                                event.stopPropagation();
                            }
                        } else {
                            break;
                        }
                    }
                }
            } else {
                throw new Error("The event given was not an instance of the emitters \"Event\" class.");
            }
        };
        
        this.createServer = function(){
            if (!server){
                server = http.createServer(function(req, res){
                    var event = new eventServer.Event("request");
                    event.request = req;
                    event.response = res;
                
                    event.emit();
                
                    if (event.propagation){
                        res.writeHead(404);
                        res.end();
                    }
                });
            }
            return this;
        };
        
        this.listen = function(){
            if (server){
                server.listen.apply(server, arguments);
            } else {
                throw new Error("No server to listen with yet.");
            }
            return this;
        };
        
        this.close = function(){
            if (server){
                server.close.apply(server, arguments);
            } else {
                throw new Error("No server to stop.");
            }
            return this;
        };
        
        this.__defineGetter__('httpServer', function(){
            return server;
        });
        
        this.on("request", function( e ){
            var request = e.request;
            var response = e.response;
            
            //Parse Url to be an object url.
            var url = new URL('http://'+(request.headers.host)+request.url);
        
            //Branch for extension or non-extension.
            if (url.extension){
            
                /*
             * Trigger beforeResponse event.
             * This doesn't give the event the response Object,
             * to ensure there is nothing placed there that is 
             * giving responses. 
             */
             
                var preEvent = new eventServer.Event("beforeResponse");
                preEvent.request = request;
                preEvent.url = url;
            
                //Catch any errors with beforeResponse and handle them then continue on.
                try {
                    eventServer.emit(preEvent);
                } catch(e){
                    var brError = new eventServer.Event("beforeResponseError");
                    brError.url = url;
                    brError.request = request;
                
                    try {
                        eventServer.emit(brError);
                    } catch(e){
                    //Nothing for now.
                    }
                }
            
                /*
             * Main Response for request happens here. Example would be a ".jpg"
             * event, or a ".html" event.
             */
            
                var event = new eventServer.Event(url.extension);
                event.url = url;
                event.request = request;
                event.response = response;
            
                //Catch any errors in response and trigger a responseError
                try {
                    eventServer.emit(event);
                    if (!event.propagation){
                        e.stopPropagation();
                    }
                } catch (e){
                    var rError = new eventServer.Event("responseError");
                    rError.request = request;
                    rError.response = response;
                    rError.url = url;
                }
                
                //Post response event    
                var postEvent = new eventServer.Event("afterResponse");
                postEvent.request = request;
                postEvent.url = url;
                  
                try {
                    eventServer.emit(postEvent);
                } catch(e){
                    var abrError = new eventServer.Event("afterResponseError");
                    abrError.url = url;
                    abrError.request = request;
                
                    try {
                        eventServer.emit(abrError);
                    } catch(e){
                    //Nothing for now.
                    }
                }     
            
                
            } else {
                // Any url that doesn't have an extension is triggered here. Further
                // Augmentation expected here.
                var neEvent = new eventServer.Event("noExtension");
                neEvent.url = url;
                neEvent.request = request;
                neEvent.response = response;
            
                try {
                    eventServer.emit(neEvent);
                    if (!neEvent.propagation){
                        e.stopPropagation();
                    }
                } catch(e){
                    var noExtError = new eventServer.Event("noExtensionError");
                    noExtError.url = url;
                    noExtError.request = request;
                    noExtError.error = e;
                    
                    try {
                        eventServer.emit(noExtError);
                    } catch(e){
                    //Nothing for now.
                    }
                }  
            }
        
    
        });
    };
    
    EventServer.prototype = new EventEmitter();
    
    return EventServer;
});