define([
    'EventEmitter',
    'EventServer',
    'http',
    'querystring',
    'fs'],
    function( 
        EventEmitter,
        EventServer,
        http,
        querystring,
        fs){

        var NetOS = function( options ){
            if (!(this instanceof NetOS)){
                return new NetOS();
            }
        
            options = options || {};
        
            var netOS = this;
            var host = null;
            var port = null;
            var server = new EventServer();
            var serverFile;
            var listeners = this.listeners = {};
            var blacklistHash = {};
            var root = options.root || require.nodeRequire.resolve('NetOS').replace(/NetOS.js$/ig, "");
            
            var Event = function( type, onTweenEmit, onFinishEmit ){
                if (!(this instanceof Event)){
                    return new Event(type);
                }
                this.type = type;
                this.onTweenEmit = onTweenEmit || function(){};
                this.onFinishEmit = onFinishEmit || function(){};
            };
            
            var localEmit = function(event){
                //console.log("event type \""+event.type+"\" emitted.");
                if (event.type){
                    var listenerArray = listeners[event.type];
                    if (listenerArray){
                        for (var x = 0 ; x < listenerArray.length; x++){
                            listenerArray[x](event);
                        }
                    }
                } else {
                    throw new Error("The event given did not have a type.");
                }
            };
       
       
            this.nodes = [];
            this.key = "password" || options.password;
        
            server.createServer();
        
            netOS.__defineGetter__('listeners', function(){
                return listeners;
            });
        
            netOS.__defineGetter__('host', function(){
                return host;
            });
        
            netOS.__defineGetter__('port', function(){
                return port;
            });
        
            netOS.__defineGetter__('eventServer', function(){
                return server;
            });
        
            netOS.__defineGetter__('httpServer', function(){
                return server.httpServer;
            });
        
            netOS.__defineGetter__('Event', function(){
                return Event;
            });
        
            var incrBlacklist = function( host ){
                if (!blacklistHash[host]){
                    blacklistHash[host] = {
                        count: 0
                    };
                }
                blacklistHash[host].count++;

                if (blacklistHash[host].count > 2){
                    var i= netOS.nodes.indexOf(host);
                    netOS.nodes.splice(i,1);
                
                    //console.log("Host "+host+" removed because of failures to connect.");
                    writeNodesToFile();
                }
            
            };
        
            var removeFromBlacklist = function( host ){
                if (blacklistHash[host]){
                    delete blacklistHash[host];
                }
            };
        
            var writeNodesToFile = function(){
                fs.writeFile(serverFile, JSON.stringify(netOS.nodes), function(err){
                    if (!err){
                    //console.log("\"nodes("+host+"-"+port+").json\" updated to: "+JSON.stringify(netOS.nodes));
                    } else {
                        console.log("Bigger problem, so the process is going to die.");
                        throw err;
                    }
                });
            };
        
            this.emitNode = function(hostname, event){
                var onFinishEmit = event.onFinishEmit;
                var onTweenEmit = event.onTweenEmit;
                delete event.onFinishEmit;
                delete event.onTweenEmit;
            
                var req;
                var host;
                var port;
                var array;
                var options;
                var curEvent = event;
                
                array = hostname.split(":");
                host = array[0];
                port = array[1] || "80";
                       
                       
                options = {
                    host: host,
                    port: port,
                    path: '/emit',
                    method: 'PUT'
                };

                req = http.request(options, function(res) {
                    var d = "";
                    res.on('data', function(chunk){
                        d+=chunk;
                    });
                    res.on('end', function (chunk) {
                        try {
                            curEvent = JSON.parse(d);
                        } catch(e){
                        }
                    
                        onTweenEmit( curEvent );
                        onFinishEmit( curEvent );
                    });
                });

                req.on('error', function(e) {
                    incrBlacklist(hostname);
                });

                // write data to request body
                req.write(querystring.stringify({
                    event:JSON.stringify(event),
                    key: netOS.key
                }));
                        
                req.end();
                       
            };
            
            //Emitter
            this.emit = function( event ){
                var propagation = true;
                var curEvent = event;
                var onFinishEmit = event.onFinishEmit;
                var onTweenEmit = event.onTweenEmit;
                var nodes = netOS.nodes.slice(0);
                delete event.onFinishEmit;
                delete event.onTweenEmit;
            
                var throttle = function(x, cond){
                    var req;
                    var host;
                    var port;
                    var array;
                    var options;
                    var d;
                    
                    //Make the stopPropagtion a function so no response servers may 
                    //stop propagation, just the sender.
                    if (cond(x)){
                        array = nodes[x].split(":");
                        host = array[0];
                        port = array[1] || "80";
                       
                       
                        options = {
                            host: host,
                            port: port,
                            path: '/emit',
                            method: 'PUT'
                        };

                        req = http.request(options);
                        
                        req.on('response', function(res) {
                            d = "";
                            res.on('data', function(chunk){
                                d+=chunk;
                            });
                            res.on('end', function (chunk) {
                                try {
                                    curEvent = JSON.parse(d);
                                } catch(e){
                                }
                                
                                onTweenEmit( curEvent );
                                
                            });
                        });
                        req.on('error', function(e) {
                            incrBlacklist(nodes[x]);
                        });

                        // write data to request body
                        req.write(querystring.stringify({
                            event:JSON.stringify(event),
                            key: netOS.key
                        }));
                        
                        req.end();
                       
                        setTimeout(function(){
                            throttle(x+1, cond);
                        }, 0);
                    
                    }
                };
                
                throttle(0, function(x){
                    if (x < nodes.length && propagation){
                        return true;
                    } else {
                        onFinishEmit( curEvent );
                        return false;
                    }
                });
                
                return {
                    stop: function(){
                        propagation = false;
                    }
                };
            
            };
            

        
            this.listen = function( p, h ){
                var c = arguments[2] || function(){};
                
                var callback = function(){
                    port = p;
                    host = h;
                    serverFile = root+"/nodes("+host+"-"+port+").json";
                
                    fs.readFile(serverFile, function(err, data){
                        if (!err){
                            try{
                                var n = JSON.parse(data.toString());
                                if ( Object.prototype.toString.call( n ) === '[object Array]'){
                                
                                    netOS.nodes = n;
                                }
                            
                            } catch (e){
                            //Nothing for now.
                            }
                        } 
                   
                        if (netOS.nodes.indexOf(netOS.host+":"+netOS.port)<0){
                            netOS.nodes.push(netOS.host+":"+netOS.port);
                            
                        }
                        
                        c();
                    });
                };
               
                
                if (arguments.length > 2 && typeof arguments[2] === 'function') {
                    Array.prototype.splice.apply(arguments, [arguments.length-1, 1, callback]);
                } else {
                    Array.prototype.splice.apply(arguments, [arguments.length-1, 1, callback]);
                }
            
                if (arguments.length < 2 || (arguments.length === 2 || typeof arguments[1] === 'function' ) ){
                    throw new Error("You must specify a host, for NetOS to work properly.");
                }
            
                if (server){
                    server.listen.apply(server, arguments);
                } else {
                    throw new Error("No server to listen with yet.");
                }
                return server;
            };
        
            this.close = function(){
                if (server){
                    server.close.apply(server, arguments);
                } else {
                    throw new Error("No server to stop.");
                }
                return server;
            };
        
            server.on("noExtension", function(e){
                var url = e.url;
                var request = e.request;
                var response = e.response;
                
                var getParams = function( callback ){
                    var body = "";
                    request.on('data', function(data){
                        body+=data;
                    });
                    request.on('end', function(){
                        callback(querystring.parse(body));
                    });
                };

                if (url.path.indexOf('emit') === 0 ){
                    getParams(function(params){
                        
                        try {
                            var event = JSON.parse(params.event);
                        
                            if (event){
                                localEmit.apply(netOS, [event]); 
                            }
                        
                            for (var x in event){
                                if ( typeof event[x] === 'function' ){
                                    delete event[x];
                                }
                            }
                            
                            response.writeHead(200,{
                                "Content-Type":"application/json"
                            });
                            response.end(JSON.stringify(event));
                        } catch(e){
                            
                        }
                    });
                    
                    return false;
                }
            });
        
            server.on("noExtension", function(e){
                var url = e.url;
                var request = e.request;
                var response = e.response;
                var getParams = function( callback ){
                    var body = "";
                    request.on('data', function(data){
                        body+=data;
                    });
                    request.on('end', function(){
                        callback(querystring.parse(body));
                    });
                };
                
                if (url.path.indexOf('nodes') === 0 ){
                    if (request.method === 'PUT'){
                        getParams(function(params){
                            if (params.host){
                                if (netOS.nodes.indexOf(params.host)<0){
                                    netOS.nodes.push(params.host);
                                }
                                response.writeHead(200,{
                                    "Content-Type":"application/json"
                                });
                                response.end(JSON.stringify(netOS.nodes));
                                var event = new netOS.Event("nodesChange");
                                event.nodes = netOS.nodes;
                                netOS.emit(event);
                            }
                        });
                        return false;
                    } else if (request.method === 'DELETE'){
                        getParams(function(params){
                            if (params.host){
                                var i = netOS.nodes.indexOf(params.host);
                                if ( i > -1 ){
                                    netOS.nodes.splice(i,1);
                                }
                                response.writeHead(200,{
                                    "Content-Type":"application/json"
                                });
                                response.end(JSON.stringify(netOS.nodes));
                            }
                        });
                        return false;
                    } else if (request.method === 'GET'){
                        response.writeHead(200,{
                            "Content-Type":"application/json"
                        });
                        response.end(JSON.stringify(netOS.nodes));
                        return false;
                    }
                }
            });
            
            netOS.on("nodesChange", function( e ){
                var n = e.nodes;
                if (Object.prototype.toString.call( n ) === '[object Array]'){
                    //Throttling to not jam a node process.
                    var throttle = function(x, cond){
                        var nodes = netOS.nodes;
                        if (cond(x)){
                        
                            if (nodes.indexOf(n[x])<0){
                                nodes.push(n[x]);
                            }
                        
                            setTimeout(function(){
                                throttle(x+1, cond);
                            }, 0);
                        } else {
                            writeNodesToFile();
                        }
                    };
                
                    throttle(0, function(x){
                        return x < n.length;
                    });
                
                }
            });
        
        };
    
        NetOS.prototype = new EventEmitter();
    
        return NetOS;
    });